'部材3D弾塑性振動応答解析V33の標準ﾓｼﾞｭｰﾙ        ［作成］黒田英夫(2023/04)
Imports System.Drawing.Printing

Module Bars3EPVRCom
    'トラス、ラーメン、トラス・ラーメン混合の３次元弾塑性振動応答解析
    'トラス・・・節点が自由回転
    'ラーメン・・・節点が固定接続
    '「部材３次元体解析」Bars3Dの構造データを利用
    'ＭＫ型およびＭＣＫ型で振動外力への応答変化を計算
    'ＭＫ型：Ｋｘ＝λＭｘ
    'ＭＣＫ型：μ^2Ｍｚ＋μＣｚ＋Ｋｚ＝０
    'ε・σ物性は、弾性と塑性が１区分ずつ

    'V21での改良点・・・各画面でボタンを読みやすい色に改善
    'V22での改良点・・・３次元グラフィック図で回転操作性の改善
    'V23での改良点・・・トラス・ラーメン混合で鉛直方向以外の分布荷重に対する計算バグを修正
    'V24での改良点・・・応力図の変数項目の増加、最大・最小値の表示内容の変更、
    '                   σfminの応力図のバグを修正
    'V25での改良点・・・Visual Basic 2017に変更
    'V26での改良点・・・プログラムファイルの文字コードをUTF-8に統一、
    '                   データファイルの文字コードを標準でUTF-8に変更
    '                   （従来のShift-JISもオプションで可能）
    'V27での改良点・・・文字コードUTF-8の構造データ読み込みと応力図作画のバグを修正、
    '                   ３次元グラフィック図で回転操作後の部分拡大の角度を保持
    'V28での改良点・・・データ表画面で表計算ソフトから貼り付け時に次行に0が入るのを防止
    'V29での改良点・・・入力データのファイル印刷メニューを廃止
    '                   （文字コードによる不具合のため）
    'V30での改良点・・・計算時の節点付番の改良で各座標軸方向のバンド幅を算出
    '                   （これまでは最長方向の座標軸を採用）
    'V31での改良点・・・構造図と応力図で部材の線分幅を縮小、
    '                   構造図画面で条件設定部の表示を２区分から３区分に改善
    'V32での改良点・・・部材３次元体解析の入力データファイルで拘束節点の形式変更に対応
    'V33での改良点・・・フリーソフト化、VB2022に移行、文字コードはUTF-8のみ採用

    Public Const Dpai As Single = 6.283185 '2π
    Public Const Pai As Single = 3.141593  'π

    Public CalcRes$                 '計算結果表示データ
    Public Cr$                      'CR+LFのコード
    Public DirOpen$                 '立ち上げ時のディレクトリー名
    Public Fgraph$                  'ｸﾞﾗﾌﾃﾞｰﾀの出力ﾌｧｲﾙ名
    Public Fm$                      '入力データのファイル名
    Public Fmat$            '材料データのファイル名
    Public Fstruct$                 'Bars3D構造ファイル名
    Public Fy$                      '計算結果のファイル名
    Public Fy2$                     '一時保存ﾌｧｲﾙ
    Public Title$                   'タイトル
    Public TitleM$          '材料タイトル

    Public Timst As Date            '計算開始時刻

    Public InOut As Integer         'ファイル入出力の種類（0:入力、1:出力）
    Public KdataBackup As Integer   '入力データのバックアップファイル作成（0:なし、1:有り）
    Public KdataChange As Integer   'データ変更の有無（0:なし、1:有り）
    Public KE1 As Integer           '開始要素番号
    Public KE2 As Integer           '最終要素番号
    Public KfileOK As Integer       'ﾌｧｲﾙのOK有無（0:なし、1:有り）
    Public KindDamp As Integer      '減衰設定区分（0:比例減衰、1:個別設定）
    Public KindDPL As Integer       '変形区分（0:小変形、1:大変形）
    '    Public KindEncod As Integer     'ファイルの文字コード（0:UTF-8、1:Shif-JIS）
    Public KindGraph As Integer     'ｸﾞﾗﾌ種類（1:１節点多変数、2:多節点１変数）
    Public KindGrav As Integer      '自重計算の有無（0:なし、1:有り）
    Public KindLvib As Integer      '振動外力の種類（0:力、1:加速度）
    Public KindPost As Integer      '後処理の種類（1:変位図、2:応力図、
    '                                3:ｸﾞﾗﾌﾃﾞｰﾀの作成、4:ｱﾆﾒｰｼｮﾝ、
    '                                5:周波数分析、6:端数処理、7:主要値表示）
    Public KindSLoad As Integer     '静荷重の有無（0:なし、1:有り）
    Public KindVEqua As Integer     '方程式種類（0:ＭＫ型、1:ＭＣＫ型）
    Public KmouseDchange As Integer 'ﾏｳｽ指定要素のﾃﾞｰﾀ変更有無
    Public KmouseElem As Integer    'ﾏｳｽ指定の要素番号
    Public KmousePoint As Integer   'ﾏｳｽ指定の節点番号
    Public KnewUFileA As Integer    '最近使用ﾌｧｲﾙAの更新有無
    Public KnewUFileB As Integer    '最近使用ﾌｧｲﾙBの更新有無
    Public KnewUFileC As Integer    '最近使用ﾌｧｲﾙBの更新有無
    Public KnumSb As Integer        'ﾌｫﾝﾄ文字ｻｲｽﾞの操作変数
    Public KnumSize As Integer      '構造作画での番号ｻｲｽﾞ
    Public Kscreen As Integer       '作画画面サイズ（0：全画面、1：通常画面）
    Public KStage As Integer        'ﾃﾞｰﾀ入力の段階番号
    Public Kstep As Integer         '指定の時間ﾒｯｼｭ番号
    Public Kvalue As Integer        '作画の変数ｲﾝﾃﾞｯｸｽ
    Public KvalueG As Integer       'ｸﾞﾗﾌﾃﾞｰﾀの変数ｲﾝﾃﾞｯｸｽ
    Public MdataChange As Integer   '材料データ変更の有無（0:なし、1:有り）
    Public Mprop As Integer         '材料データの成分数(=14)
    Public MultiColor As Integer    '材料別色分け（0:なし、1:有り）
    Public Nall As Integer          '変数の総数
    Public Nbound As Integer        '拘束節点数
    Public Ndamp As Integer         'ダンパーの数
    Public Ndim As Integer          '１節点当りの変数の数(=6)
    Public NEdraw As Integer        '作画要素の数
    Public Nelem As Integer         '要素数
    Public Nforce As Integer        '荷重節点数
    Public NLvib As Integer         '振動外力の荷重個数
    Public Nmat As Integer          '材料データ数
    Public NMdivMax As Integer      '材料の物性区分数最大値
    Public Norder As Integer        '正方行列の次数
    Public NPgraph As Integer       'ｸﾞﾗﾌ節点指定（節点番号または節点数）
    Public Npoint As Integer        '節点数
    Public NPvib As Integer         '振動外力節点数
    Public Nspring As Integer       'バネ支持個数
    Public Nstep As Integer         '出力時間間隔の倍数（1以上の整数）
    Public NstepF As Integer        '振動外力時間間隔の倍数（1以上の整数）
    Public Nsurf As Integer         '表面分布荷重数
    Public Ntype As Integer         '問題のタイプ（1:トラス、2:ラーメン、3:両者混合）
    Public Nvar As Integer          '１要素当りの変数の総数
    Public NvibTime As Integer      '振動外力の時間メッシュ数

    Public Cfix As Single           'XY方向ﾋﾟｸｾﾙ差の補正係数
    Public CoeffK As Single         '質量比例の減衰係数
    Public CoeffM As Single         '剛性比例の減衰係数
    Public DispRatio As Single      '変位作画の倍率
    Public DistFactor As Single     '視点とモデルとの距離倍率（標準1）
    Public DTime As Single          '振動応答計算の時間間隔
    Public EyeAngle As Single       '視点と原点を結ぶ線の水平面との角度
    Public Eyeh As Single           '視点高さ
    Public HFactor As Single        '視点の高さ倍率（基準1）
    Public RateSigma As Single      '曲げ応力の評価係数
    Public RateTau As Single        '剪断応力の評価係数
    Public Rlenmax As Single        'ﾓﾃﾞﾙの1軸の最大長さ
    Public Rlenmodel As Single      '視点の距離（座標系間）
    Public TimeMax As Single        '打切り時間
    Public TimeNew As Single        '現在の計算時刻
    Public XaxisMax As Single       'Ｘ軸の最大値
    Public XaxisMin As Single       'Ｘ軸の最小値
    Public Xlower As Single         '作画領域のX最小値
    Public XRotate As Single        'モデルのX軸回りの回転角（右ねじ正、°）
    Public Xupper As Single         '作画領域のX最大値
    Public XYscale As Single        '作画スケール
    Public YaxisMax As Single       'Ｙ軸の最大値
    Public YaxisMin As Single       'Ｙ軸の最小値
    Public Ylower As Single         '作画領域のY最小値
    Public YRotate As Single        'モデルのY軸回りの回転角（右ねじ正、°）
    Public Yupper As Single         '作画領域のY最大値
    Public ZRotate As Single        'モデルのZ軸回りの回転角（右ねじ正、°）

    Public UFileA(6) As String      '最近使用した構造ﾌｧｲﾙの保存配列
    Public UFileB(6) As String      '最近使用した解析ﾌｧｲﾙの保存配列
    Public UFileC(6) As String      '最近使用した材料ﾌｧｲﾙの保存配列

    Public Jdirect(,) As Integer    '節点拘束の各軸方向拘束有無(0:非拘束、1:拘束）
    Public JLoadVib(,) As Integer   '節点振動外力の各軸方向の有無
    Public Kcomp() As Integer       '元ﾃﾞｰﾀ節点の計算変数順序
    Public Kdispmb() As Integer     '全体行列の変数番号対応の変位方向
    Public KdispMatR() As Integer   '計算行列の変数番号対応の変位方向
    Public KdrawElem() As Integer   '各要素の作画有無（0:なし、1:有り）
    Public KEorder() As Integer     '作画順の要素番号
    Public KEsurf() As Integer      '表面分布荷重の要素番号
    Public KindElem() As Integer    '要素種類（1:トラス、2:ラーメン、3:左節点回転自由、
    '                                4:右節点回転自由）
    Public KindMises() As Integer   'Mises応力区分（0:第1入力点以下、1:左記超え）
    Public KPdamp(,) As Integer     'ダンパーの2節点番号
    Public KPforce() As Integer     '節点荷重の節点番号
    Public KPgraph(20) As Integer   'グラフ節点番号
    Public KPspring(,) As Integer   'バネ支持の節点番号
    Public KPvib() As Integer       '振動外力の節点番号
    Public KRcomp() As Integer      '計算順序節点の元ﾃﾞｰﾀ節点順序
    Public LrD() As Integer         '元ﾃﾞｰﾀ節点の第1変数の元全体ﾏﾄﾘｯｸｽ番号
    Public NMdiv() As Integer       '材料の物性区分数
    Public NNelem(,) As Integer     '各要素における全節点番号
    Public Nodemb() As Integer      '元全体行列の変数番号対応の節点番号
    Public NodeMatR() As Integer    '計算行列の変数番号対応の節点順位
    Public NPrefer() As Integer     '要素計算における節点変数の対応番号
    Public Nrest() As Integer       '拘束の節点番号
    Public NumMat() As Integer      '各要素の材料種類番号
    Public NvarPD() As Integer      '元ﾃﾞｰﾀ節点の変数の数

    Public AngleYBar() As Single    '断面Y軸の角度（最上向きから左回り、°）
    Public AxisMax(3) As Single     'モデルのXYZ各軸の最大値
    Public AxisMin(3) As Single     'モデルのXYZ各軸の最小値
    Public Cmatrix() As Single      'Ｃ行列
    Public CenterPos(3) As Single   '構造中心の座標
    Public CosYBar(,) As Single     '部材Y軸の全体X,Y,Z軸に対する方向余弦
    Public Cspring(,) As Single     'バネ定数
    Public DataDamp(,) As Single    'ダンパーの外部点座標と減衰係数
    Public DataMat(,) As Single     '各材料の物性値
    Public EpsMat(,) As Single      '材料の区分歪み
    Public FLvibTime(,) As Single   '各時間毎の振動外力
    Public Force(,) As Single       '節点荷重値（各軸方向）
    Public Fspring(,) As Single     '初期バネ力（各軸方向）
    Public GravMulti(3) As Single   '自重のX,Y,Z方向の倍率
    Public PnormYB() As Single      '部材Y座標軸方向の垂直分布荷重（単位長さ当り）
    Public PnormZB() As Single      '部材Z座標軸方向の垂直分布荷重（単位長さ当り）
    Public Ptang() As Single        '表面の接線方向分布荷重（単位長さ当り）
    Public RMmatrix() As Single     'Ｍ行列（対角要素のみ）
    Public RouMat() As Single       '各材料の密度
    Public SigmaMat(,) As Single    '材料の区分応力
    Public Tdisp(,) As Single       '結果の変位
    Public TransMat(4, 4) As Single 'モデルの合成アフィン変換行列
    Public Tu() As Single           'FEM計算の解
    Public Vdirect(,) As Single     '節点拘束の各軸方向拘束値
    Public Xdata(), Ydata(), Zdata() As Single  '節点座標
    Public XE(2), YE(2), ZE(2) As Single    '要素計算の配列
    Public Xmodel(), Ymodel(), Zmodel() As Single   'モデル座標における節点座標
    Public Xmodel2(), Ymodel2(), Zmodel2() As Single   'モデル座標における節点座標
    Public Xmouse(2), Ymouse(2) As Single   'ﾏｳｽ位置座標
    Public Xnew(), Ynew(), Znew() As Single   '変位入りの節点座標
    Public Xscreen(), Yscreen() As Single   'スクリーンにおける節点座標
    Public Xscreen2(), Yscreen2() As Single   'スクリーンにおける節点座標
    Public Xview(), Yview(), Zview() As Single  '視点座標における節点座標（左手系）
    Public Xview2(), Yview2(), Zview2() As Single  '視点座標における節点座標（左手系）

    Sub AdjustRatio(ByRef ratio As Single)
        '変位比率の調整
        Dim nb As Integer
        Dim b1, b2, std As Single

        std = 0.05
        b1 = std / ratio
        If b1 < 1.0 Then
            ratio = 1.0
        ElseIf b1 < 10.0 Then
            b2 = b1 * 10
            ratio = Int(b2) / 10.0
        Else    '上位2桁を採用
            b2 = b1
            nb = 0
            Do
                b2 = b2 / 10
                nb = nb + 1
                If b2 < 10 Then
                    Exit Do
                End If
            Loop
            b2 = Int(b2 * 10)
            If nb < 2 Then
                ratio = b2
            Else
                ratio = b2 * 10 ^ (nb - 1)
            End If
        End If

    End Sub

    Sub AxisCheck(ByVal k1 As Integer, ByVal k2 As Integer, _
ByRef rlenmaxb As Single, ByRef xd() As Single, _
ByRef yd() As Single, ByRef zd() As Single)
        'X、Y、Z軸の最小・最大値のﾁｪｯｸ
        'k1,k2･･･要素の開始、終了番号
        'rlenmaxb･･･1軸の最大長さ
        'xd(),yd(),zd()･･･節点の座標
        Dim ib, ie, ii, ik, j As Integer
        Dim b1 As Single
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 2
                    ib = NNelem(ie, j)
                    If ik = 0 Then
                        ik = 1
                        AxisMin(1) = xd(ib)
                        AxisMin(2) = yd(ib)
                        AxisMin(3) = zd(ib)
                        For ii = 1 To 3
                            AxisMax(ii) = AxisMin(ii)
                        Next ii
                    Else
                        For ii = 1 To 3
                            Select Case ii
                                Case 1
                                    b1 = xd(ib)
                                Case 2
                                    b1 = yd(ib)
                                Case 3
                                    b1 = zd(ib)
                            End Select
                            If b1 < AxisMin(ii) Then
                                AxisMin(ii) = b1
                            ElseIf b1 > AxisMax(ii) Then
                                AxisMax(ii) = b1
                            End If
                        Next ii
                    End If
                Next j
            End If
        Next ie

        rlenmaxb = AxisMax(1) - AxisMin(1)
        For ii = 2 To 3
            b1 = AxisMax(ii) - AxisMin(ii)
            If b1 > rlenmaxb Then
                rlenmaxb = b1
            End If
        Next ii

    End Sub

    Sub AxisCheckS(ByVal k1 As Integer, ByRef k2 As Integer, ByRef xmin As Single, _
    ByRef xmax As Single, ByRef ymin As Single, ByRef ymax As Single, _
    ByRef xdb() As Single, ByRef ydb() As Single)
        'X･Y座標の最小・最大値のﾁｪｯｸ
        'k1,k2･･･要素の開始、終了番号
        'xmin･･･X座標最小値
        'xmax･･･X座標最大値
        'ymin･･･Y座標最小値
        'ymax･･･Y座標最大値
        'xdb(),ydb()･･･節点座標
        Dim i, ie, ik, j As Integer
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 2
                    i = NNelem(ie, j)
                    If ik = 0 Then
                        ik = 1
                        xmin = xdb(i) : xmax = xmin
                        ymin = ydb(i) : ymax = ymin
                    Else
                        If xdb(i) < xmin Then
                            xmin = xdb(i)
                        ElseIf xdb(i) > xmax Then
                            xmax = xdb(i)
                        End If
                        If ydb(i) < ymin Then
                            ymin = ydb(i)
                        ElseIf ydb(i) > ymax Then
                            ymax = ydb(i)
                        End If
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub AxisSet(ByRef tdb(,) As Single)
        '座標の設定
        Dim i As Integer
        For i = 1 To Npoint
            Xnew(i) = Xdata(i) + DispRatio * tdb(i, 1)
            Ynew(i) = Ydata(i) + DispRatio * tdb(i, 2)
            Znew(i) = Zdata(i) + DispRatio * tdb(i, 3)
        Next i
    End Sub

    Sub AxisValueTreat(ByRef xbuf() As Single, ByRef ybuf() As Single, ByRef zbuf() As Single)
        '座標値の端数処理
        'xbuf(),ybuf(),zbuf()･･･節点の座標
        Dim i, ib, ie, kind, nb As Integer
        Dim b2, rlenb As Single
        ReDim KdrawElem(Nelem)

        For ie = 1 To Nelem
            KdrawElem(ie) = 1
        Next ie

        Call AxisCheck(1, Nelem, rlenb, Xdata, Ydata, Zdata)
        'X、Y、Z軸の最小・最大値のﾁｪｯｸ

        b2 = rlenb * 0.00002
        nb = 0
        i = Int(b2)
        If i > 0 Then
            kind = 0
            Do
                ib = CInt(b2)
                If ib < 10 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 0.1
                End If
            Loop
        Else
            kind = 1
            Do
                ib = CInt(b2)
                If ib > 0 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 10
                End If
            Loop
        End If

        For i = 1 To Npoint
            xbuf(i) = Xdata(i)
            ybuf(i) = Ydata(i)
            zbuf(i) = Zdata(i)
        Next i
        For i = 1 To Npoint
            If Xdata(i) <> 0 Then
                Call RoundTreat(kind, nb, xbuf(i)) '丸め処理
            End If
            If Ydata(i) <> 0 Then
                Call RoundTreat(kind, nb, ybuf(i)) '丸め処理
            End If
            If Zdata(i) <> 0 Then
                Call RoundTreat(kind, nb, zbuf(i)) '丸め処理
            End If
        Next i

    End Sub

    Sub CfileInputU(ByVal fxx$, ByRef kh As Integer, ByVal kstepx As Integer)
        '計算結果ﾌｧｲﾙの読込み
        'UTF-8･･･BOMなし
        'fxx$･･･計算結果ﾌｧｲﾙ名
        'kh･･･ﾃﾞｰﾀ有無（0:なし、1:有り）
        'kstepx･･･指定ｽﾃｯﾌﾟ番号（-1の場合は計算条件部分）
        Dim aa$
        Dim istep As Integer

        aa$ = ""
        kh = 0
        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String
        If kstepx < 0 Then
            Do
                aa$ = sr.ReadLine
                If aa$ = "＜パラメーター条件＞" Then
                    kh = 1
                    Exit Do
                End If
            Loop
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            TimeMax = dat(1)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            DTime = dat(1)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            NstepF = dat(1)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            Nstep = dat(1)
        Else
            Do
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                If dat(0) = "＜時間メッシュ番号、時間＞" Then
                    istep = dat(1)
                    TimeNew = dat(2)
                    If istep = kstepx Then
                        kh = 1
                        Exit Do
                    End If
                End If
            Loop
        End If

        sr.Close()

    End Sub

    Sub ColorSet(ByVal kb As Integer, ByRef colorx As Color)
        '色の設定
        'kb･･･指定番号（1～）
        'colorx･･･色
        Dim kcount As Integer
        kcount = (kb - 1) Mod 24
        Select Case kcount
            Case 0
                colorx = Color.Black
            Case 1
                colorx = Color.Green
            Case 2
                colorx = Color.DarkOrange
            Case 3
                colorx = Color.FromArgb(0, 0, 128)
            Case 4
                colorx = Color.FromArgb(128, 0, 0)
            Case 5
                colorx = Color.FromArgb(192, 99, 0)
            Case 6
                colorx = Color.Violet
            Case 7
                colorx = Color.DarkBlue
            Case 8
                colorx = Color.FromArgb(90, 90, 90)
            Case 9
                colorx = Color.DarkGreen
            Case 10
                colorx = Color.DarkRed
            Case 11
                colorx = Color.Brown
            Case 12
                colorx = Color.FromArgb(0, 128, 99)
            Case 13
                colorx = Color.FromArgb(192, 99, 99)
            Case 14
                colorx = Color.LimeGreen
            Case 15
                colorx = Color.Purple
            Case 16
                colorx = Color.Gray
            Case 17
                colorx = Color.FromArgb(99, 99, 192)
            Case 18
                colorx = Color.FromArgb(160, 128, 0)
            Case 19
                colorx = Color.FromArgb(160, 128, 99)
            Case 20
                colorx = Color.BurlyWood
            Case 21
                colorx = Color.DarkSeaGreen
            Case 22
                colorx = Color.DarkKhaki
            Case 23
                colorx = Color.DarkOrchid
        End Select

    End Sub

    Sub CoordiChange(ByVal rlenb As Single)
        '節点座標の視点座標への変換
        'rlenb･･･ﾓﾃﾞﾙ座標原点と視点座標との距離
        Dim ip As Integer
        For ip = 1 To Npoint
            Xview(ip) = Xmodel(ip)
            Yview(ip) = Zmodel(ip)
            Zview(ip) = Ymodel(ip) + rlenb
        Next ip

    End Sub

    Private Sub CrosPoint(ByRef alphab As Single, ByRef beta As Single, _
 ByVal eps As Single, ByVal xbuf() As Single, ByVal ybuf() As Single, _
 ByVal xbufc() As Single, ByVal ybufc() As Single)
        '2線分の交点計算
        'alphab,beta･･･各線分の交点位置比
        'xbuf(),ybuf()･･･基準側の2節点座標
        'xbufc(),ybufc()･･･比較側の2節点座標
        Dim a1, a2, b1, b2, b3, b4, bb, cc, dd, dist, distb As Single
        Dim eps2, f1, f2, xb, yb As Single

        alphab = -1
        eps2 = eps * eps
        a1 = ybuf(2) - ybuf(1)
        b1 = -(xbuf(2) - xbuf(1))
        dist = Math.Sqrt(a1 * a1 + b1 * b1)
        a2 = ybufc(2) - ybufc(1)
        b2 = -(xbufc(2) - xbufc(1))
        distb = Math.Sqrt(a2 * a2 + b2 * b2)
        If dist < eps Or distb < eps Then
            Exit Sub
        End If
        If Math.Abs(a1) < eps And Math.Abs(b2) < eps Then
            xb = xbufc(1)
            yb = ybuf(1)
            b1 = xb - xbuf(1)
            b2 = yb - ybufc(1)
            cc = b1 * (xb - xbuf(2))
            dd = b2 * (yb - ybufc(2))
            If cc < eps2 And dd < eps2 Then
                alphab = Math.Abs(b1) / dist
                beta = Math.Abs(b2) / distb
            End If
        ElseIf Math.Abs(b1) < eps And Math.Abs(a2) < eps Then
            xb = xbuf(1)
            yb = ybufc(1)
            b1 = yb - ybuf(1)
            b2 = xb - xbufc(1)
            cc = b1 * (yb - ybuf(2))
            dd = b2 * (xb - xbufc(2))
            If cc < eps2 And dd < eps2 Then
                alphab = Math.Abs(b1) / dist
                beta = Math.Abs(b2) / distb
            End If
        Else
            f1 = a1 * xbuf(1) + b1 * ybuf(1)
            f2 = a2 * xbufc(1) + b2 * ybufc(1)
            bb = a1 * b2 - b1 * a2
            If Math.Abs(bb) > eps2 Then
                xb = (f1 * b2 - f2 * b1) / bb
                yb = (f2 * a1 - f1 * a2) / bb
                b1 = xb - xbuf(1)
                b2 = yb - ybuf(1)
                cc = b1 * (xb - xbuf(2))
                dd = b2 * (yb - ybuf(2))
                If cc < eps2 And dd < eps2 Then
                    b3 = xb - xbufc(1)
                    b4 = yb - ybufc(1)
                    cc = b3 * (xb - xbufc(2))
                    dd = b4 * (yb - ybufc(2))
                    If cc < eps2 And dd < eps2 Then
                        alphab = Math.Sqrt(b1 * b1 + b2 * b2) / dist
                        beta = Math.Sqrt(b3 * b3 + b4 * b4) / distb
                    End If
                End If
            End If
        End If

    End Sub

    Sub DataStoreCheck(ByRef ix As Integer)
        'ﾃﾞｰﾀ保存の確認
        'ix･･･返答（6:yes、それ以外は中止）
        ix = 0
        If KdataChange > 0 Then
            Dim aa$
            aa$ = Fm$ + "ファイルが変更されていますが、保存なしでよいですか？"
            ix = MsgBox(aa$, 4, "データ変更の警告")
            If ix = 6 Then
                KdataChange = 0
            End If
        End If

    End Sub

    Sub Dfile2InputU(ByVal fxx$)
        '構造ﾃﾞｰﾀﾌｧｲﾙの読み込み
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾌｧｲﾙ名
        Dim aa$
        Dim i, ib, id, ie, im, ip, j, jb As Integer

        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String

        aa$ = sr.ReadLine
        dat = aa.Split(",")
        If dat(0) <> "プログラム名" Then
            aa$ = "本ソフトのUTF-8データファイルとは違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        If dat(1) <> "Bars3D" Then
            aa$ = "データファイルが違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        Title$ = sr.ReadLine
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Ntype = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Npoint = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nelem = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nbound = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nmat = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nforce = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nsurf = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nspring = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        KindGrav = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Ndim = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Mprop = dat(1)

        ReDim KindElem(Nelem), NNelem(Nelem, 2), NumMat(Nelem), CosYBar(Nelem, 3)
        ReDim AngleYBar(Nelem)
        ReDim Xdata(Npoint), Ydata(Npoint), Zdata(Npoint)
        ReDim Nrest(Nbound), Jdirect(Nbound, 6), Vdirect(Nbound, 6)
        ReDim DataMat(Nmat, Mprop)
        ReDim KPforce(Nforce), Force(Nforce, 6)
        ReDim KEsurf(Nsurf)
        ReDim KPspring(Nspring, 2), Cspring(Nspring, 3), Fspring(Nspring, 3)
        ReDim PnormYB(Nsurf), PnormZB(Nsurf), Ptang(Nsurf)

        aa$ = sr.ReadLine
        If Ntype < 3 Then
            For ie = 1 To Nelem
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                NumMat(ie) = dat(1)
                For j = 1 To 2
                    NNelem(ie, j) = dat(j + 1)
                Next j
                AngleYBar(ie) = dat(4)
            Next ie
        Else
            For ie = 1 To Nelem
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                KindElem(ie) = dat(1)
                NumMat(ie) = dat(2)
                For j = 1 To 2
                    NNelem(ie, j) = dat(j + 2)
                Next j
                AngleYBar(ie) = dat(5)
            Next ie
        End If

        aa$ = sr.ReadLine
        For ip = 1 To Npoint
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            Xdata(ip) = dat(1)
            Ydata(ip) = dat(2)
            Zdata(ip) = dat(3)
        Next ip

        If Nbound > 0 Then
            aa$ = sr.ReadLine
            If Len(aa$) < 10 Then
                For i = 1 To Nbound
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    Nrest(i) = dat(3)
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    For j = 1 To 3
                        jb = 4 * (j - 1)
                        Jdirect(i, j) = dat(jb + 1)
                        Vdirect(i, j) = dat(jb + 3)
                    Next j
                    If Ndim > 3 Then
                        aa$ = sr.ReadLine
                        dat = aa.Split(",")
                        For j = 4 To Ndim
                            jb = 4 * (j - 4)
                            Jdirect(i, j) = dat(jb + 1)
                            Vdirect(i, j) = dat(jb + 3)
                        Next j
                    End If
                Next i
            Else
                For i = 1 To Nbound
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    Nrest(i) = dat(1)
                    For j = 1 To Ndim
                        jb = 2 * j
                        Jdirect(i, j) = dat(jb)
                        Vdirect(i, j) = dat(jb + 1)
                    Next j
                Next i
            End If
        End If

        aa$ = sr.ReadLine
        For im = 1 To Nmat
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            For j = 1 To Mprop
                DataMat(im, j) = dat(j)
            Next j
        Next im

        If Nforce > 0 Then
            aa$ = sr.ReadLine
            For ib = 1 To Nforce
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                KPforce(ib) = dat(1)
                For id = 1 To Ndim
                    Force(ib, id) = dat(id + 1)
                Next id
            Next ib
        End If

        If Nsurf > 0 Then
            aa$ = sr.ReadLine
            For ib = 1 To Nsurf
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                KEsurf(ib) = dat(1)
                PnormYB(ib) = dat(2)
                PnormZB(ib) = dat(3)
                Ptang(ib) = dat(4)
            Next ib
        End If

        If Nspring > 0 Then
            aa$ = sr.ReadLine
            For i = 1 To Nspring
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                For j = 1 To 2
                    KPspring(i, j) = dat(j)
                Next j
                For j = 1 To 3
                    jb = 2 * j
                    Cspring(i, j) = dat(jb + 1)
                    Fspring(i, j) = dat(jb + 2)
                Next j
            Next i
        End If

        If KindGrav > 0 Then
            aa$ = sr.ReadLine
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            For i = 1 To 3
                GravMulti(i) = dat(i - 1)
            Next i
        End If

        sr.Close()

    End Sub

    Sub DfileInputU(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙの読み込み
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾌｧｲﾙ名
        Dim aa$
        Dim i, ib, id, j As Integer

        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String

        aa$ = sr.ReadLine
        dat = aa.Split(",")
        If dat(0) <> "プログラム名" Then
            aa$ = "本ソフトのUTF-8データファイルとは違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        If dat(1) <> "Bars3EPVibRes" Then
            aa$ = "データファイルが違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Fstruct$ = dat(1)
        Title$ = sr.ReadLine
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Ntype = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Npoint = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        KindVEqua = dat(1)
        If KindVEqua = 0 Then
            KindDamp = 0
            Ndamp = 0
        Else
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            KindDamp = dat(1)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            Ndamp = dat(1)
        End If
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        ib = dat(1)
        If ib > 0 Then
            aa$ = "周波数応答には対応していません！"
            MsgBox(aa$, 0, "入力データエラー")
            Exit Sub
        End If
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        ib = dat(1)
        If ib > 0 Then
            aa$ = "拘束拘束の解除には対応していません！"
            MsgBox(aa$, 0, "入力データエラー")
            Exit Sub
        End If
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Norder = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nmat = dat(1)

        ReDim RouMat(Nmat)
        ReDim NodeMatR(Norder), KdispMatR(Norder)

        If Ndamp > 0 Then
            ReDim KPdamp(Ndamp, 2)
            ReDim DataDamp(Ndamp, 4)
            aa$ = sr.ReadLine
            For i = 1 To Ndamp
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                KPdamp(i, 1) = dat(1)
                KPdamp(i, 2) = dat(2)
                For j = 1 To 4
                    DataDamp(i, j) = dat(j + 2)
                Next j
            Next i
        End If

        aa$ = sr.ReadLine
        For i = 1 To Nmat
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            RouMat(i) = dat(1)
        Next i

        aa$ = sr.ReadLine
        For i = 1 To Norder
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            NodeMatR(i) = dat(1)
            KdispMatR(i) = dat(2)
        Next i

        ReDim RMmatrix(Norder)
        aa$ = sr.ReadLine
        For i = 1 To Norder
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            RMmatrix(i) = dat(1)
        Next i

        If KindVEqua > 0 Then
            Select Case KindDamp
                Case 0
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    CoeffM = dat(1)
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    CoeffK = dat(1)
                Case 1
                    ReDim Cmatrix(Norder)
                    aa$ = sr.ReadLine
                    For i = 1 To Norder
                        aa$ = sr.ReadLine
                        dat = aa.Split(",")
                        Cmatrix(i) = dat(1)
                    Next i
            End Select
        End If

        If Ntype = 1 Then
            Ndim = 3
        Else
            Ndim = 6
        End If

        aa$ = sr.ReadLine
        aa$ = sr.ReadLine
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        NPvib = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        NLvib = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        NvibTime = dat(1)
        ReDim FLvibTime(NvibTime, NLvib)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        KindLvib = dat(1)
        ReDim KPvib(NPvib), JLoadVib(NPvib, Ndim)

        aa$ = sr.ReadLine
        For i = 1 To NPvib
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            KPvib(i) = dat(1)
            For id = 1 To Ndim
                JLoadVib(i, id) = dat(id + 1)
            Next id
        Next i

        aa$ = sr.ReadLine
        For i = 0 To NvibTime
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            For j = 1 To NLvib
                FLvibTime(i, j) = dat(j)
            Next j
        Next i

        sr.Close()

    End Sub

    Sub DfileOutU(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙへの書き出し
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾃﾞｰﾀﾌｧｲﾙ名
        Dim a1$, aa$, ad$
        Dim i, ib, id, j As Integer

        a1$ = ""
        Dim sw As New IO.StreamWriter(fxx$)

        aa$ = "プログラム名" + "," + "Bars3EPVibRes"
        sw.WriteLine(aa$)
        aa$ = "［Bars3D構造ﾌｧｲﾙ］" + "," + Fstruct$
        sw.WriteLine(aa$)
        sw.WriteLine(Title$)
        aa$ = "問題のタイプ" + "," + Str(Ntype)
        sw.WriteLine(aa$)
        aa$ = "節点数" + "," + Str(Npoint)
        sw.WriteLine(aa$)
        aa$ = "方程式種類(0:MK型、1:MCK型)" + "," + Str(KindVEqua)
        sw.WriteLine(aa$)
        If KindVEqua = 0 Then
            KindDamp = 0
            Ndamp = 0
        Else
            aa$ = "減衰設定区分(0:比例減衰、1:個別設定)" + "," + Str(KindDamp)
            sw.WriteLine(aa$)
            aa$ = "ダンパー数" + "," + Str(Ndamp)
            sw.WriteLine(aa$)
        End If
        ib = 0
        aa$ = "解析の種類（0:一般応答、1:周波数応答）" + "," + Str(ib)
        sw.WriteLine(aa$)
        aa$ = "境界拘束の解除（0:なし、1:有り）" + "," + Str(ib)
        sw.WriteLine(aa$)
        aa$ = "行列の次数" + "," + Str(Norder)
        sw.WriteLine(aa$)
        aa$ = "材料データ数" + "," + Str(Nmat)
        sw.WriteLine(aa$)

        If Ndamp > 0 Then
            aa$ = "＜ダンパーデータ＞No.、節点番号1、節点番号2、外部点X、Y、Z、減衰係数"
            sw.WriteLine(aa$)
            For i = 1 To Ndamp
                aa$ = Str$(i) + ",  " + Str$(KPdamp(i, 1)) + ",  " + Str$(KPdamp(i, 2))
                For j = 1 To 4
                    aa$ = aa$ + ",  " + Str$(DataDamp(i, j))
                Next j
                sw.WriteLine(aa$)
            Next i
        End If

        sw.WriteLine("＜材料の密度＞材料番号、密度")
        For i = 1 To Nmat
            aa$ = Str$(i) + ",  " + Str$(RouMat(i))
            sw.WriteLine(aa$)
        Next i

        aa$ = "＜行列の変数番号対応＞変数番号、節点番号、変位方向"
        sw.WriteLine(aa$)
        For i = 1 To Norder
            aa$ = Str$(i) + ",  " + Str$(NodeMatR(i))
            aa$ = aa$ + ",  " + Str$(KdispMatR(i))
            sw.WriteLine(aa$)
        Next i

        sw.WriteLine("＜Ｍ行列（質量マトリックス）＞行番号、対角要素")
        For i = 1 To Norder
            aa$ = Str$(i) + ",  "
            aa$ = aa$ + Str$(RMmatrix(i))
            sw.WriteLine(aa$)
        Next i

        If KindVEqua = 1 Then
            Select Case KindDamp
                Case 0
                    aa$ = "質量比例の減衰係数" + "," + Str(CoeffM)
                    sw.WriteLine(aa$)
                    aa$ = "剛性比例の減衰係数" + "," + Str(CoeffK)
                    sw.WriteLine(aa$)
                Case 1
                    sw.WriteLine("＜Ｃ行列（減衰マトリックス）＞行番号、各要素")
                    For i = 1 To Norder
                        aa$ = Str$(i) + ",  "
                        aa$ = aa$ + Str$(Cmatrix(i))
                        sw.WriteLine(aa$)
                    Next i
            End Select
        End If

        ad$ = "-------------------------------------------------------"
        sw.WriteLine(ad$)
        sw.WriteLine("＜振動外力データ＞")
        aa$ = "振動外力の節点数" + "," + Str(NPvib)
        sw.WriteLine(aa$)
        aa$ = "振動外力の荷重個数" + "," + Str(NLvib)
        sw.WriteLine(aa$)
        aa$ = "振動外力の時間メッシュ数" + "," + Str(NvibTime)
        sw.WriteLine(aa$)
        aa$ = "振動外力の種類（0:力、1:加速度）" + "," + Str(KindLvib)
        sw.WriteLine(aa$)
        If Ntype = 1 Then
            aa$ = "No.、節点番号、各方向有無（X、Y、Z）"
        Else
            aa$ = "No.、節点番号、各方向有無（X、Y、Z、θx、θy、θz）"
        End If
        sw.WriteLine(aa$)
        For i = 1 To NPvib
            aa$ = Str$(i) + ",  " + Str$(KPvib(i))
            For id = 1 To Ndim
                aa$ = aa$ + ",   " + Str$(JLoadVib(i, id))
            Next id
            sw.WriteLine(aa$)
        Next i

        aa$ = "時間No."
        For i = 1 To NPvib
            ib = KPvib(i)
            For id = 1 To Ndim
                j = JLoadVib(i, id)
                If j > 0 Then
                    Select Case id
                        Case 1
                            a1$ = "u"
                        Case 2
                            a1$ = "v"
                        Case 3
                            a1$ = "w"
                        Case 4
                            a1$ = "θx"
                        Case 4
                            a1$ = "θx"
                        Case 4
                            a1$ = "θy"
                        Case 4
                            a1$ = "θz"
                    End Select
                    a1$ = a1$ + LTrim$(Str$(ib))
                    aa$ = aa$ + "、" + a1$
                End If
            Next id
        Next i
        sw.WriteLine(aa$)
        For i = 0 To NvibTime
            aa$ = Str$(i)
            For j = 1 To NLvib
                aa$ = aa$ + ",  " + Str$(FLvibTime(i, j))
            Next j
            sw.WriteLine(aa$)
        Next i

        sw.Close()

        aa$ = "データをファイルへ保存しました。"
        MsgBox(aa$, 0, "ファイル保存完了")

    End Sub

    Function DispMax(ByVal k1 As Integer, ByRef k2 As Integer, _
    ByRef tdb(,) As Single) As Single
        '変位の最大値
        'k1,k2･･･要素の開始、終了番号
        'dmax･･･X座標最小値
        'tdb(,)･･･節点の各座標方向変位
        Dim i, ie, j, k As Integer
        Dim b1, dmax As Single
        dmax = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 2
                    i = NNelem(ie, j)
                    For k = 1 To 3
                        b1 = Math.Abs(tdb(i, k))
                        If b1 > dmax Then
                            dmax = b1
                        End If
                    Next k
                Next j
            End If
        Next ie
        Return dmax

    End Function

    Sub DispRateAnim(ByRef ratio As Single, ByVal rlen2 As Single)
        '変位作画倍率の設定
        'ratio･･･変位作画の倍率
        'rlen2･･･最大変位
        Dim rlen As Single

        Call AxisCheck(KE1, KE2, rlen, Xdata, Ydata, Zdata)
        '座標の最小・最大値のﾁｪｯｸ

        ratio = rlen2 / rlen
        Call AdjustRatio(ratio)     '変位比率の調整

    End Sub

    Sub DispRateSet(ByRef ratio As Single, ByVal tfdb(,) As Single)
        '変位作画倍率の設定
        'ratio･･･変位作画の倍率
        'tfdb(,)･･･節点の各座標変位
        Dim rlen, rlen2 As Single

        Call AxisCheck(KE1, KE2, rlen, Xdata, Ydata, Zdata)
        '座標の最小・最大値のﾁｪｯｸ

        rlen2 = DispMax(KE1, KE2, tfdb)
        ratio = rlen2 / rlen
        Call AdjustRatio(ratio)     '変位比率の調整

    End Sub

    Private Sub DistComp(ByRef ic As Integer, ByVal k1 As Integer, ByVal k2 As Integer)
        '距離比較
        'ic･･･距離区分（0:近い、1:遠い）
        'k1･･･基準要素番号
        'k2･･･比較要素番号
        Dim i1, i2, j1, j2 As Integer
        Dim alphab, b1, b2, b3, b4, beta, dist, distc, eps, xb, yb, zb As Single
        Dim xbuf(2), xbufc(2), ybuf(2), ybufc(2) As Single

        ic = -1  '仮設定
        i1 = NNelem(k1, 1)
        i2 = NNelem(k1, 2)
        xbuf(1) = Xscreen(i1)
        ybuf(1) = Yscreen(i1)
        xbuf(2) = Xscreen(i2)
        ybuf(2) = Yscreen(i2)
        b1 = xbuf(1) - xbuf(2)
        b2 = ybuf(1) - ybuf(2)
        b3 = b1 * b1 + b2 * b2

        j1 = NNelem(k2, 1)
        j2 = NNelem(k2, 2)
        xbufc(1) = Xscreen(j1)
        ybufc(1) = Yscreen(j1)
        xbufc(2) = Xscreen(j2)
        ybufc(2) = Yscreen(j2)
        b1 = xbufc(1) - xbufc(2)
        b2 = ybufc(1) - ybufc(2)
        b4 = b1 * b1 + b2 * b2
        If b4 > b3 Then
            b3 = b4
        End If
        b3 = 0.0001 * Math.Sqrt(b3)

        Call CrosPoint(alphab, beta, eps, xbuf, ybuf, xbufc, ybufc)
        '2要素の交点計算

        If alphab >= 0 Then
            b1 = 1 - alphab
            xb = b1 * Xmodel(i1) + alphab * Xmodel(i2)
            yb = b1 * Ymodel(i1) + alphab * Ymodel(i2) + Rlenmodel
            zb = b1 * Zmodel(i1) + alphab * Zmodel(i2) - Eyeh
            dist = xb * xb + yb * yb + zb * zb
            b1 = 1 - beta
            xb = b1 * Xmodel(j1) + beta * Xmodel(j2)
            yb = b1 * Ymodel(j1) + beta * Ymodel(j2) + Rlenmodel
            zb = b1 * Zmodel(j1) + beta * Zmodel(j2) - Eyeh
            distc = xb * xb + yb * yb + zb * zb
            If dist < distc Then
                ic = 0
            Else
                ic = 1
            End If
        End If

    End Sub

    Sub DrawFocusRect(ByRef gr As Graphics)
        'ﾌｫｰｶｽ矩形の描画
        Dim rect As Rectangle
        Dim p1, p2 As Point

        If Xmouse(1) <= Xmouse(2) Then
            p1.X = Xmouse(1)
            p2.X = Xmouse(2)
        Else
            p1.X = Xmouse(2)
            p2.X = Xmouse(1)
        End If
        If Ymouse(1) <= Ymouse(2) Then
            p1.Y = Ymouse(1)
            p2.Y = Ymouse(2)
        Else
            p1.Y = Ymouse(2)
            p2.Y = Ymouse(1)
        End If
        rect.X = p1.X
        rect.Y = p1.Y
        rect.Width = p2.X - p1.X
        rect.Height = p2.Y - p1.Y

        gr.DrawRectangle(Pens.DarkGray, rect)   '矩形の描画

    End Sub

    Sub DrawScale(ByVal dpixb As Single, ByVal dpiyb As Single, _
        ByVal xlen As Single, ByVal ylen As Single, ByVal xmin As Single, _
        ByVal xmax As Single, ByVal ymin As Single, ByVal ymax As Single)
        '作画倍率の計算
        'dpixb･･･X軸方向のdpi値
        'dpiyb･･･Y軸方向のdpi値
        'xlen･･･作画領域のX方向長さ
        'ylen･･･作画領域のY方向長さ
        'xmin･･･X座標最小値
        'xmax･･･X座標最大値
        'ymin･･･Y座標最小値
        'ymax･･･Y座標最大値
        Dim b1, b2, cx, cy As Single

        Cfix = dpiyb / dpixb

        b1 = xmax - xmin : b2 = ymax - ymin
        If Cfix < 1 Then
            If b1 <> 0 Then
                cx = xlen / b1
            End If
            If b2 <> 0 Then
                cy = ylen / b2 / Cfix
            End If
        Else
            If b1 <> 0 Then
                cx = Cfix * xlen / b1
            End If
            If b2 <> 0 Then
                cy = ylen / b2
            End If
        End If
        If b1 = 0 Then
            XYscale = cy
        ElseIf b2 = 0 Then
            XYscale = cx
        ElseIf cy < cx Then
            XYscale = cy
        Else
            XYscale = cx
        End If

    End Sub

    Sub Elem3DPrep()
        '構造図作画準備
        Dim ie As Integer
        ReDim KdrawElem(Nelem)
        ReDim Xmodel(Npoint), Ymodel(Npoint), Zmodel(Npoint)
        ReDim Xscreen(Npoint), Yscreen(Npoint)
        ReDim Xview(Npoint), Yview(Npoint), Zview(Npoint)

        KE1 = 1
        KE2 = Nelem
        KnumSize = 8
        For ie = KE1 To KE2
            KdrawElem(ie) = 1
        Next ie

        Rlenmax = 0
        DistFactor = 0      '作画ﾊﾟﾗﾒｰﾀｰのｸﾘｱ

    End Sub

    Sub ElemSearch(ByRef k1 As Integer, ByRef k2 As Integer, _
    ByRef xsb() As Single, ByRef ysb() As Single)
        '拡大作画する要素の探索
        '両節点が指定領域にある要素を採用、作画要素なしの場合は元のﾃﾞｰﾀを保存
        'k1,k2･･･要素の開始、終了番号
        'xsb(),ysb()･･･節点のｽｸﾘｰﾝ座標
        Dim ib, ic, ie, j, kmax, kmin As Integer
        Dim b1, b2 As Single
        Dim kbuf(Nelem) As Integer

        ic = 0
        For ie = 1 To Nelem
            kbuf(ie) = KdrawElem(ie)
            If KdrawElem(ie) > 0 Then
                For j = 1 To 2
                    ib = NNelem(ie, j)
                    b1 = xsb(ib) - Xmouse(1)
                    b2 = xsb(ib) - Xmouse(2)
                    If b1 >= 0 And b2 <= 0 Then
                        b1 = ysb(ib) - Ymouse(2)
                        b2 = ysb(ib) - Ymouse(1)
                        If b1 >= 0 And b2 <= 0 Then
                            If j = 2 Then
                                KdrawElem(ie) = 1
                                If ic = 0 Then
                                    ic = 1
                                    kmin = ie
                                    kmax = ie
                                ElseIf ie < kmin Then
                                    kmin = ie
                                ElseIf ie > kmax Then
                                    kmax = ie
                                End If
                            End If
                        Else
                            KdrawElem(ie) = 0
                            Exit For
                        End If
                    Else
                        KdrawElem(ie) = 0
                        Exit For
                    End If
                Next j
            End If
        Next ie
        k1 = kmin
        k2 = kmax
        If k1 = 0 Then
            For ie = 1 To Nelem
                KdrawElem(ie) = kbuf(ie)
            Next ie
        End If

    End Sub

    Sub ElemSort(ByVal k1 As Integer, ByVal k2 As Integer)
        '作画要素のｿｰﾃｨﾝｸﾞ
        'k1,k2･･･開始・終了要素番号
        Dim i1, i2, ic, ie, ii, j, nb As Integer

        NEdraw = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                NEdraw = NEdraw + 1
            End If
        Next ie
        ReDim KEorder(NEdraw)
        nb = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                nb = nb + 1
                KEorder(nb) = ie
            End If
        Next ie

        For ii = 1 To NEdraw - 1
            i1 = KEorder(ii)
            For j = ii + 1 To NEdraw
                i2 = KEorder(j)
                Call DistComp(ic, i1, i2)    '距離比較
                If ic = 0 Then
                    KEorder(ii) = i2
                    KEorder(j) = i1
                    i1 = i2
                End If
            Next j
        Next ii

    End Sub

    Sub FileName(ByVal fxx$, ByRef fxb$)
        'ﾌｧｲﾙ名fxx$の拡張子なしの部分fxb$を取り出す
        Dim i As Integer
        i = InStr(fxx$, ".")
        If i > 0 Then
            fxb$ = Left$(fxx$, i - 1)
        Else
            fxb$ = fxx$
        End If
    End Sub

    Sub MatFileInputU(ByVal fxx$)
        '材料ﾃﾞｰﾀﾌｧｲﾙの読み込み
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾌｧｲﾙ名
        Dim aa$
        Dim i, j As Integer

        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String

        aa$ = sr.ReadLine
        dat = aa.Split(",")
        If dat(0) <> "プログラム名" Then
            aa$ = "本ソフトのUTF-8データファイルとは違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        If dat(1) <> "Bars3EPVibRes" Then
            aa$ = "データファイルが違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        TitleM$ = sr.ReadLine
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nmat = dat(1)
        ReDim NMdiv(Nmat)

        aa$ = sr.ReadLine
        NMdivMax = 0
        For i = 1 To Nmat
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            NMdiv(i) = dat(1)
            If NMdiv(i) > NMdivMax Then
                NMdivMax = NMdiv(i)
            End If
        Next i
        ReDim EpsMat(Nmat, NMdivMax), SigmaMat(Nmat, NMdivMax)
        aa$ = sr.ReadLine
        For i = 1 To Nmat
            aa$ = sr.ReadLine
            aa$ = sr.ReadLine
            For j = 0 To NMdiv(i)
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                EpsMat(i, j) = dat(1)
                SigmaMat(i, j) = dat(2)
            Next j
        Next i

        sr.Close()

    End Sub

    Sub MatFileOutU(ByVal fxx$)
        '材料入力ﾃﾞｰﾀﾌｧｲﾙへの書き出し
        'fxx$･･･入力ﾃﾞｰﾀﾌｧｲﾙ名
        Dim aa$
        Dim i, j As Integer

        Dim sw As New IO.StreamWriter(fxx$)

        aa$ = "プログラム名" + "," + "Bars3EPVibRes"
        sw.WriteLine(aa$)
        sw.WriteLine(TitleM$)
        aa$ = "材料データ数" + "," + Str(Nmat)
        sw.WriteLine(aa$)
        sw.WriteLine("各材料の物性区分数（材料番号、区分数)")
        For i = 1 To Nmat
            aa$ = "  " + Str$(i) + ",  " + Str$(NMdiv(i))
            sw.WriteLine(aa$)
        Next i
        sw.WriteLine("各材料の歪み・応力データ")
        For i = 1 To Nmat
            aa$ = "材料番号" + "," + Str(i)
            sw.WriteLine(aa$)
            sw.WriteLine("  区分番号、歪み、応力")
            For j = 0 To NMdiv(i)
                aa$ = "    " + Str$(j) + ",  " + Str$(EpsMat(i, j))
                aa$ = aa$ + ",  " + Str$(SigmaMat(i, j))
                sw.WriteLine(aa$)
            Next j
        Next i

        sw.Close()

        aa$ = "データをファイルへ保存しました。"
        MsgBox(aa$, 0, "ファイル保存完了")

    End Sub

    Sub ModelTrans(ByVal nb As Integer, ByRef xbuf() As Single, _
ByRef ybuf() As Single, ByRef zbuf() As Single)
        'ﾓﾃﾞﾙ節点座標のｱﾌｨﾝ変換
        'nb･･･節点数
        'xbuf(),ybuf(),zbuf()･･･節点のX,Y,Z座標
        Dim ip, j, k As Integer
        Dim b1 As Single
        Dim buf(3) As Single
        For ip = 1 To nb
            buf(1) = xbuf(ip)
            buf(2) = ybuf(ip)
            buf(3) = zbuf(ip)
            For k = 1 To 3
                b1 = 0
                For j = 1 To 3
                    b1 = b1 + TransMat(k, j) * buf(j)
                Next j
                b1 = b1 + TransMat(k, 4)
                Select Case k
                    Case 1
                        xbuf(ip) = b1
                    Case 2
                        ybuf(ip) = b1
                    Case 3
                        zbuf(ip) = b1
                End Select
            Next k
        Next ip

    End Sub

    Sub MouseCalc()
        'ﾏｳｽ点のﾓﾃﾞﾙ座標への変換
        Dim j As Integer
        Dim xa, xb, ya, yb As Single
        For j = 1 To 2
            xa = Xmouse(j)
            ya = Ymouse(j)
            If Cfix < 1 Then
                xb = (xa - Xlower) / XYscale + XaxisMin
                yb = (Yupper - ya) / XYscale / Cfix + YaxisMin
            Else
                xb = Cfix * (xa - Xlower) / XYscale + XaxisMin
                yb = (Yupper - ya) / XYscale + YaxisMin
            End If
            Xmouse(j) = xb
            Ymouse(j) = yb
        Next j

    End Sub

    Sub MouseCalcS(ByVal xa As Single, ByRef xb As Single, ByVal ya As Single, ByRef yb As Single)
        'ﾏｳｽ点のﾓﾃﾞﾙ座標への変換
        'xa,ya･･･ﾏｳｽ点座標
        'xb,yb･･･ﾓﾃﾞﾙ座標
        If Cfix < 1 Then
            xb = (xa - Xlower) / XYscale + XaxisMin
            yb = (Yupper - ya) / XYscale / Cfix + YaxisMin
        Else
            xb = Cfix * (xa - Xlower) / XYscale + XaxisMin
            yb = (Yupper - ya) / XYscale + YaxisMin
        End If

    End Sub

    Sub MousePointSearch(ByVal k1 As Integer, ByVal k2 As Integer, _
    ByRef kp As Integer, ByVal xb As Single, ByVal yb As Single, _
    ByRef xmb() As Single, ByRef ymb() As Single)
        'ﾏｳｽ点の節点の探索
        'k1,k2･･･要素の開始、終了番号
        'kp･･･ﾏｳｽ点の節点番号
        'xb,yb･･･ﾏｳｽ点の作画座標位置
        'xmb(),ymb()･･･節点のｽｸﾘｰﾝ座標
        Dim ib, ie, j As Integer
        Dim b1, b2, b3, rlen2 As Single
        Dim kbuf(Npoint) As Integer

        kp = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 2
                    ib = NNelem(ie, j)
                    If kbuf(ib) = 0 Then
                        kbuf(ib) = 1
                        b1 = xmb(ib) - xb
                        b2 = ymb(ib) - yb
                        b3 = b1 * b1 + b2 * b2
                        If kp = 0 Or b3 < rlen2 Then
                            kp = ib
                            rlen2 = b3
                        End If
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub NvarPDSet()
        '元ﾃﾞｰﾀ節点の変数の数を設定
        Dim i, ib, ii, ip, j, k, nb As Integer

        For i = 1 To Npoint
            NvarPD(i) = 3
        Next i
        For i = 1 To Nelem
            If KindElem(i) > 1 Then
                For j = 1 To 2
                    k = NNelem(i, j)
                    NvarPD(k) = 6
                Next j
            End If
        Next i

        '行列の変数番号の対応設定
        ii = 0
        For ip = 1 To Npoint
            nb = NvarPD(ip)
            ib = 0
            LrD(ip) = ii + 1
            For j = 1 To nb
                ii = ii + 1
                ib = ib + 1
                Nodemb(ii) = ip
                Kdispmb(ii) = ib
            Next j
        Next ip

    End Sub

    Sub OptFileIn()
        'ｵﾌﾟｼｮﾝ･ﾌｧｲﾙ読み込み
        Dim ftmp$

        ftmp$ = DirOpen$ + "\Bars3EPVRMenu.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            KdataBackup = Val(sr.ReadLine)
            Kscreen = Val(sr.ReadLine)
            MultiColor = Val(sr.ReadLine)
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub OptFileOut()
        'ｵﾌﾟｼｮﾝ･ﾌｧｲﾙ書き出し
        Dim ftmp$

        ftmp$ = DirOpen$ + "\Bars3EPVRMenu.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        sw.WriteLine(KdataBackup)
        sw.WriteLine(Kscreen)
        sw.WriteLine(MultiColor)
        sw.Close()

    End Sub

    Sub PerspecCalc(ByVal k1 As Integer, ByVal k2 As Integer, ByVal rlenscreen As Single, _
    ByRef xvbuf() As Single, ByRef yvbuf() As Single, ByRef zvbuf() As Single, _
    ByRef xsbuf() As Single, ByRef ysbuf() As Single)
        'ｽｸﾘｰﾝへの透視座標の計算
        'k1,k2･･･要素の開始、終了番号
        'rlenscreen･･･透視座標原点からｽｸﾘｰﾝまでの距離
        'xvbuf(),yvbuf(),zvbuf()･･･視線座標のX,Y,Z
        'xsbuf(),ysbuf()･･･ｽｸﾘｰﾝ座標のX,Y
        Dim ib, ie, j As Integer
        Dim b1 As Single
        Dim kbuf(Npoint) As Integer

        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 2
                    ib = NNelem(ie, j)
                    If kbuf(ib) = 0 Then
                        b1 = rlenscreen / zvbuf(ib)
                        xsbuf(ib) = b1 * xvbuf(ib)
                        ysbuf(ib) = b1 * (yvbuf(ib) - Eyeh)
                        kbuf(ib) = 1
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub ResFileScan(ByVal ii As Integer)
        '結果ﾌｧｲﾙの読み飛ばし
        'ii･･･ﾌｧｲﾙ番号
        Dim aa$
        Dim i As Integer

        aa$ = ""
        Do
            Input(ii, aa$)
            If aa$ = "＜変位データ＞" Then
                Exit Do
            End If
        Loop
        Input(ii, aa$)
        For i = 1 To Npoint
            aa$ = LineInput(ii)
        Next i
        Input(ii, aa$) : Input(ii, aa$) : Input(ii, aa$)
        For i = 1 To 2 * Nelem
            aa$ = LineInput(ii)
        Next i
        Input(ii, aa$) : Input(ii, aa$) : Input(ii, aa$)
        For i = 1 To Npoint
            aa$ = LineInput(ii)
        Next i

    End Sub

    Sub ResFileScanU(ByVal sr As IO.StreamReader)
        '結果ﾌｧｲﾙの読み飛ばし
        'UTF-8･･･BOMなし
        Dim aa$
        Dim i As Integer

        aa$ = ""
        Do
            aa$ = sr.ReadLine
            If aa$ = "＜変位データ＞" Then
                Exit Do
            End If
        Loop
        aa$ = sr.ReadLine
        For i = 1 To Npoint
            aa$ = sr.ReadLine
        Next i
        aa$ = sr.ReadLine
        aa$ = sr.ReadLine
        aa$ = sr.ReadLine
        For i = 1 To 2 * Nelem
            aa$ = sr.ReadLine
        Next i
        aa$ = sr.ReadLine
        aa$ = sr.ReadLine
        aa$ = sr.ReadLine
        For i = 1 To Npoint
            aa$ = sr.ReadLine
        Next i

    End Sub

    Sub RotateCalc(ByVal kind As Integer, ByRef theta As Single)
        '回転角の計算
        'kind･･･象限番号
        'theta･･･回転角（左ねじ正、°）
        Dim ix, iy, kb As Integer
        Dim b1 As Single

        ix = Xmouse(2) - Xmouse(1)
        iy = Ymouse(2) - Ymouse(1)
        Select Case kind
            Case 1
                If ix > 0 Then
                    If iy >= 0 Then
                        kb = -1
                    Else
                        kind = 0
                    End If
                Else
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = 1
                    End If
                End If
            Case 2
                If ix > 0 Then
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = -1
                    End If
                Else
                    If iy >= 0 Then
                        kb = 1
                    Else
                        kind = 0
                    End If
                End If
            Case 3
                If ix > 0 Then
                    If iy >= 0 Then
                        kb = 1
                    Else
                        kind = 0
                    End If
                Else
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = -1
                    End If
                End If
            Case 4
                If ix > 0 Then
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = 1
                    End If
                Else
                    If iy >= 0 Then
                        kb = -1
                    Else
                        kind = 0
                    End If
                End If
        End Select
        b1 = Math.Sqrt(ix * ix + iy * iy)
        theta = kb * 3.0 * b1 / 50.0

    End Sub

    Sub RoundPrep(ByRef kind As Integer, ByRef nb As Integer, ByVal zb As Single)
        '丸め準備
        'kind･･･丸め区分（0:1以上、1:1未満）
        'nb･･･丸め位置の桁数
        'zb･･･丸め基準値
        Dim i, ib As Integer
        Dim b2 As Single

        b2 = zb
        nb = 0
        i = Int(b2)
        If i > 0 Then
            kind = 0
            Do
                ib = CInt(b2)
                If ib < 10 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 0.1
                End If
            Loop
        Else
            kind = 1
            Do
                ib = CInt(b2)
                If ib > 0 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 10
                End If
            Loop
        End If

    End Sub

    Sub RoundTreat(ByVal kind As Integer, ByVal nb As Integer, ByRef zb As Single)
        '丸め処理
        'kind･･･丸め区分（0:1以上、1:1未満）
        'nb･･･丸め位置の桁数
        'zb･･･対象変数
        Dim ib As Integer
        Dim b1 As Single

        If kind = 0 Then
            b1 = zb * 10 ^ (-nb)
            ib = CInt(b1)
            zb = ib * 10 ^ nb
        Else
            b1 = zb * 10 ^ nb
            ib = CInt(b1)
            zb = ib * 10 ^ (-nb)
        End If

    End Sub

    Sub TimCal(ByVal ktim As Integer, ByRef timn$)
        'ﾀｲﾏｰ計算
        'ktim･･･時間継続指定（0:時間ｸﾘｱする、1:時間継続する）
        'timn$･･･時分秒に換算した経過時間
        Dim rtime As Date = Now
        Dim rhour, rmin, sec As Single
        If ktim > 0 Then
            sec = DateDiff(DateInterval.Second, Timst, rtime)
            rhour = Int(sec / 3600) : sec = sec - rhour * 3600
            rmin = Int(sec / 60) : sec = Int(sec - rmin * 60)
            timn$ = Str$(rhour) + "h" + Str$(rmin) + "min" + Str$(sec) + "s"
        Else
            Timst = rtime
        End If
    End Sub

    Sub TmatMulti(ByRef tr1buf(,) As Single, ByRef tr2buf(,) As Single)
        '変換行列の掛算
        'tr1buf(,)･･･最初の変換行列
        'tr2buf(,)･･･次の変換行列および結果の変化行列
        Dim i, j, k As Integer
        Dim b1 As Single
        Dim buf(4, 4) As Single

        For i = 1 To 4
            For j = 1 To 4
                buf(i, j) = tr2buf(i, j)
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf(i, k) * tr1buf(k, j)
                Next k
                tr2buf(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmatrixCalD(ByVal xr As Single, ByVal yr As Single, ByVal zr As Single, _
    ByRef trbuf(,) As Single)
        'ﾓﾃﾞﾙの合成ｱﾌｨﾝ変換行列算出
        'Z,Y,X軸回りの回転
        'xr,yr,zr･･･X,Y,Z軸回りの回転角（右ねじ正、°）
        'trbuf(,)･･･変換ﾏﾄﾘｯｸｽ
        Dim i, j, k As Integer
        Dim b1, b2, thetax, thetay, thetaz As Single
        Dim buf1(4, 4), buf2(4, 4), buf3(4, 4), buf4(4, 4) As Single

        b1 = Pai / 180
        thetax = xr * b1
        thetay = yr * b1
        thetaz = zr * b1

        For i = 1 To 4
            buf1(i, i) = 1
        Next i

        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf2(1, 1) = 1
        buf2(2, 2) = b1
        buf2(2, 3) = -b2
        buf2(3, 2) = b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1

        b1 = Math.Cos(thetay)
        b2 = Math.Sin(thetay)
        buf3(1, 1) = b1
        buf3(1, 3) = b2
        buf3(2, 2) = 1
        buf3(3, 1) = -b2
        buf3(3, 3) = b1
        buf3(4, 4) = 1

        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf4(1, 1) = b1
        buf4(1, 2) = -b2
        buf4(2, 1) = b2
        buf4(2, 2) = b1
        buf4(3, 3) = 1
        buf4(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf2(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + trbuf(i, k) * buf3(k, j)
                Next k
                buf1(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf4(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmatrixCalE(ByVal xr As Single, ByVal yr As Single, ByVal zr As Single, _
    ByRef trbuf(,) As Single)
        '座標軸の合成ｱﾌｨﾝ変換行列算出
        'Z,Y,X軸回りの回転
        'xr,yr,zr･･･X,Y,Z軸回りの回転角（右ねじ正、°）
        'trbuf(,)･･･変換ﾏﾄﾘｯｸｽ
        Dim i, j, k As Integer
        Dim b1, b2, thetax, thetay, thetaz As Single
        Dim buf1(4, 4), buf2(4, 4), buf3(4, 4), buf4(4, 4) As Single

        b1 = Pai / 180
        thetax = xr * b1
        thetay = yr * b1
        thetaz = zr * b1

        For i = 1 To 4
            buf1(i, i) = 1
        Next i

        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf2(1, 1) = 1
        buf2(2, 2) = b1
        buf2(2, 3) = b2
        buf2(3, 2) = -b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1

        b1 = Math.Cos(thetay)
        b2 = Math.Sin(thetay)
        buf3(1, 1) = b1
        buf3(1, 3) = -b2
        buf3(2, 2) = 1
        buf3(3, 1) = b2
        buf3(3, 3) = b1
        buf3(4, 4) = 1

        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf4(1, 1) = b1
        buf4(1, 2) = b2
        buf4(2, 1) = -b2
        buf4(2, 2) = b1
        buf4(3, 3) = 1
        buf4(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf2(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + trbuf(i, k) * buf3(k, j)
                Next k
                buf1(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf4(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TMatrixCos(ByVal thetax As Single, ByVal thetay As Single, _
        ByVal thetaz As Single)
        '方向余弦用変換ﾏﾄﾘｯｸｽの計算
        'thetax･･･X軸回りの回転角ﾗｼﾞｱﾝ（第1ｽﾃｯﾌﾟ）
        'thetay･･･Y軸回りの回転角ﾗｼﾞｱﾝ（第2ｽﾃｯﾌﾟ）
        'thetaz･･･Z軸回りの回転角ﾗｼﾞｱﾝ（第3ｽﾃｯﾌﾟ）
        Dim i, j, k As Integer
        Dim b1, b2 As Single
        Dim buf(4, 4), buf1(4, 4), buf2(4, 4), buf3(4, 4) As Single
        ReDim TransMat(4, 4)

        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf3(1, 1) = 1
        buf3(2, 2) = b1
        buf3(2, 3) = -b2
        buf3(3, 2) = b2
        buf3(3, 3) = b1
        buf3(4, 4) = 1

        b1 = Math.Cos(thetay)
        b2 = Math.Sin(thetay)
        buf2(1, 1) = b1
        buf2(1, 3) = b2
        buf2(2, 2) = 1
        buf2(3, 1) = -b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf2(i, k) * buf3(k, j)
                Next k
                buf(i, j) = b1
            Next j
        Next i

        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf1(1, 1) = b1
        buf1(1, 2) = -b2
        buf1(2, 1) = b2
        buf1(2, 2) = b1
        buf1(3, 3) = 1
        buf1(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf(k, j)
                Next k
                TransMat(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmpFileIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$

        ftmp$ = DirOpen$ + "\Bars3EPVibRes.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            Fstruct$ = sr.ReadLine
            Fm$ = sr.ReadLine
            Fmat$ = sr.ReadLine
            Fy$ = sr.ReadLine
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub TmpFileOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$

        ftmp$ = DirOpen$ + "\Bars3EPVibRes.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        sw.WriteLine(Fstruct$)
        sw.WriteLine(Fm$)
        sw.WriteLine(Fmat$)
        sw.WriteLine(Fy$)
        sw.Close()

    End Sub

    Sub UFileAddA()
        'UFileAの追加処理
        Dim i, ic, j As Integer
        ic = 0
        For i = 2 To 6
            If UFileA(i) = UFileA(0) Then
                For j = i - 1 To 0 Step -1
                    UFileA(j + 1) = UFileA(j)
                Next j
                ic = 1
                Exit For
            End If
        Next i
        If ic = 0 Then
            For i = 5 To 0 Step -1
                UFileA(i + 1) = UFileA(i)
            Next i
        End If
        UseFileAOut()       'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し

    End Sub

    Sub UFileAddB()
        'UFileBの追加処理
        Dim i, ic, j As Integer
        ic = 0
        For i = 2 To 6
            If UFileB(i) = UFileB(0) Then
                For j = i - 1 To 0 Step -1
                    UFileB(j + 1) = UFileB(j)
                Next j
                ic = 1
                Exit For
            End If
        Next i
        If ic = 0 Then
            For i = 5 To 0 Step -1
                UFileB(i + 1) = UFileB(i)
            Next i
        End If
        UseFileBOut()       'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し

    End Sub

    Sub UFileAddC()
        'UFileCの追加処理
        Dim i, ic, j As Integer
        ic = 0
        For i = 2 To 6
            If UFileC(i) = UFileC(0) Then
                For j = i - 1 To 0 Step -1
                    UFileC(j + 1) = UFileC(j)
                Next j
                ic = 1
                Exit For
            End If
        Next i
        If ic = 0 Then
            For i = 5 To 0 Step -1
                UFileC(i + 1) = UFileC(i)
            Next i
        End If
        UseFileCOut()       'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し

    End Sub

    Sub UseFileAIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\Bars3EPVRUFileA.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            i = 0
            Do Until sr.Peek = -1 'ﾌｧｲﾙ末尾まで
                i = i + 1
                UFileA(i) = sr.ReadLine
            Loop
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub UseFileAOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\Bars3EPVRUFileA.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        For i = 1 To 6
            If UFileA(i) <> "" Then
                sw.WriteLine(UFileA(i))
            Else
                Exit For
            End If
        Next i
        sw.Close()

    End Sub

    Sub UseFileBIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\Bars3EPVRUFileB.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            i = 0
            Do Until sr.Peek = -1 'ﾌｧｲﾙ末尾まで
                i = i + 1
                UFileB(i) = sr.ReadLine
            Loop
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub UseFileBOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\Bars3EPVRUFileB.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        For i = 1 To 6
            If UFileB(i) <> "" Then
                sw.WriteLine(UFileB(i))
            Else
                Exit For
            End If
        Next i
        sw.Close()

    End Sub

    Sub UseFileCIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\Bars3EPVRUFileC.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            i = 0
            Do Until sr.Peek = -1 'ﾌｧｲﾙ末尾まで
                i = i + 1
                UFileC(i) = sr.ReadLine
            Loop
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub UseFileCOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\Bars3EPVRUFileC.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        For i = 1 To 6
            If UFileC(i) <> "" Then
                sw.WriteLine(UFileC(i))
            Else
                Exit For
            End If
        Next i
        sw.Close()

    End Sub

    Sub VCosCalc(ByRef kh As Integer)
        '断面ＹB軸の方向余弦計算
        'kh･･･ｴﾗｰ有無（0:なし、1:有り）
        Dim aa$
        Dim i, ic, ie, j, k1, k2 As Integer
        Dim alpha, b1, hpai, rlen1, rlenb As Single
        Dim thetax, thetay, thetaz As Single
        Dim buf(3), vec(4) As Single

        hpai = 1.570796
        alpha = Pai / 180
        kh = 0
        For ie = 1 To Nelem
            k1 = NNelem(ie, 1)
            k2 = NNelem(ie, 2)
            buf(1) = Xdata(k2) - Xdata(k1)
            buf(2) = Ydata(k2) - Ydata(k1)
            buf(3) = Zdata(k2) - Zdata(k1)

            'ﾃﾞｰﾀ異常のﾁｪｯｸ
            If buf(1) = 0 And buf(2) = 0 And buf(3) = 0 Then
                kh = 1
                aa$ = "要素" + Str$(ie) + "の両端節点が重なっています！"
                MsgBox(aa$, 0, "データエラー")
                Exit Sub
            End If

            thetax = -AngleYBar(ie) * alpha     '第1ｽﾃｯﾌﾟ
            If buf(3) < 0 Then  '要素を上向き方向に変換
                thetax = -thetax
                For j = 1 To 3
                    buf(j) = -buf(j)
                Next j
            End If
            '第2ｽﾃｯﾌﾟ
            ic = 0
            If buf(3) = 0 Then  '水平の場合
                thetay = 0
            ElseIf buf(1) = 0 And buf(2) = 0 Then   '垂直の場合
                thetay = hpai
                ic = 1
                thetaz = 0
            Else
                b1 = 0
                For j = 1 To 3
                    b1 = b1 + buf(j) * buf(j)
                Next j
                rlenb = Math.Sqrt(b1)     '要素長さ
                b1 = 0
                For j = 1 To 2
                    b1 = b1 + buf(j) * buf(j)
                Next j
                rlen1 = Math.Sqrt(b1)     'XY平面への投影長さ
                thetay = -Math.Atan(buf(3) / rlen1)   'X軸上での回転角
            End If
            If ic = 0 Then
                If buf(1) = 0 Then  'X座標が同じ場合
                    If buf(2) > 0 Then
                        thetaz = hpai
                    Else
                        thetaz = -hpai
                    End If
                ElseIf buf(2) = 0 Then  'Y座標が同じ場合
                    If buf(1) > 0 Then
                        thetaz = 0
                    Else
                        thetaz = pai
                    End If
                Else
                    thetaz = Math.Atan(buf(2) / buf(1))
                    If buf(1) < 0 Then
                        thetaz = thetaz + pai
                    End If
                End If
            End If
            Call TMatrixCos(thetax, thetay, thetaz)  '方向余弦用変換ﾏﾄﾘｯｸｽの計算
            vec(1) = 0
            vec(2) = 0
            vec(3) = 1  'Z軸ﾍﾞｸﾄﾙに取る
            vec(4) = 1
            For i = 1 To 3
                b1 = 0
                For j = 1 To 4
                    b1 = b1 + TransMat(i, j) * vec(j)
                Next j
                CosYBar(ie, i) = b1
            Next i
        Next ie
    End Sub

    Sub XYCalc(ByRef xb As Single, ByRef yb As Single)
        'X,Yの作画座標計算
        'xb･･･X座標値
        'yb･･･Y座標値
        Dim b1, b2 As Single
        b1 = xb - XaxisMin
        b2 = yb - YaxisMin
        If Cfix < 1 Then
            xb = b1 * XYscale + Xlower
            yb = Yupper - b2 * XYscale * Cfix
        Else
            xb = b1 * XYscale / Cfix + Xlower
            yb = Yupper - b2 * XYscale
        End If
    End Sub

End Module
